@@ -8,24 +8,20 @@ from math import comb
 from typing import List, Optional, Set
 
 import ray
-from numpy import median
 from pymatgen.analysis.phase_diagram import GrandPotentialPhaseDiagram, PhaseDiagram
 from pymatgen.entries.computed_entries import ComputedEntry
 from tqdm import tqdm
 
-from rxn_network.core import Enumerator
+from rxn_network.core.enumerator import Enumerator
 from rxn_network.entries.entry_set import GibbsEntrySet
-from rxn_network.enumerators.utils import (
-    apply_calculators,
-    group_by_chemsys,
-    initialize_calculators,
-    initialize_entry,
-)
-from rxn_network.reactions import ComputedReaction
-from rxn_network.utils import initialize_ray, limited_powerset, to_iterator
+from rxn_network.entries.utils import initialize_entry
+from rxn_network.enumerators.utils import get_rxn_info, group_by_chemsys
+from rxn_network.reactions.computed import ComputedReaction
+from rxn_network.reactions.reaction_set import ReactionSet
+from rxn_network.utils.funcs import get_logger, grouper, limited_powerset
+from rxn_network.utils.ray import initialize_ray
 
-
-PARALLEL_THRESHOLD = 4  # median computation size above which parallelization enabled
+logger = get_logger(__name__)
 
 
 class BasicEnumerator(Enumerator):
@@ -37,19 +33,21 @@ class BasicEnumerator(Enumerator):
     products may not be stable with respect to each other.
     """
 
+    CHUNK_SIZE = 2500
+
     def __init__(
         self,
         precursors: Optional[List[str]] = None,
         targets: Optional[List[str]] = None,
-        calculators: Optional[List[str]] = None,
         n: int = 2,
         exclusive_precursors: bool = True,
         exclusive_targets: bool = False,
+        filter_by_chemsys: Optional[str] = None,
+        max_num_constraints=1,
         remove_unbalanced: bool = True,
         remove_changed: bool = True,
-        calculate_e_above_hulls: bool = True,
+        calculate_e_above_hulls: bool = False,
         quiet: bool = False,
-        parallel: bool = True,
     ):
         """
         Supplied target and calculator parameters are automatically initialized as
@@ -57,56 +55,60 @@ class BasicEnumerator(Enumerator):
 
         Args:
             precursors: Optional list of precursor formulas; only reactions
-                which contain at least these phases as reactants will be enumerated. See the
-                "exclusive_precursors" parameter for more details.
+                which contain at least these phases as reactants will be enumerated. See
+                the "exclusive_precursors" parameter for more details.
             targets: Optional list of target formulas; only reactions which include
                 formation of at least one of these targets will be enumerated. See the
                 "exclusive_targets" parameter for more details.
-            calculators: Optional list of Calculator object names to be initialized; see calculators
-                module for options (e.g., ["ChempotDistanceCalculator"])
             n: Maximum reactant/product cardinality; i.e., largest possible number of
                 entries on either side of the reaction. Defaults to 2.
-            exclusive_precursors: Whether to consider only reactions that have
-            reactants which are a subset of the provided list of precursors. In other
-                words, if True, this only identifies reactions with reactants selected from the precursors
-                argument. Defaults to True.
+            exclusive_precursors: Whether to consider only reactions that have reactants
+                which are a subset of the provided list of precursors. In other
+                words, if True, this only identifies reactions with reactants selected
+                from the precursors argument. Defaults to True.
             exclusive_targets: Whether to consider only reactions that make the
-                form products that are a subset of the provided list of targets. If False,
-                this only identifies reactions with no unspecified byproducts. Defualts to False.
+                form products that are a subset of the provided list of targets. If
+                False, this only identifies reactions with no unspecified byproducts.
+                Defualts to False.
             remove_unbalanced: Whether to remove reactions which are unbalanced.
                 Defaults to True.
             remove_changed: Whether to remove reactions which can only be balanced by
                 removing a reactant/product or having it change sides. Defaults to True.
+            calculate_e_above_hulls: Whether to calculate e_above_hull for each entry
+                upon initialization of the entries at the beginning of enumeration.
             quiet: Whether to run in quiet mode (no progress bar). Defaults to False.
         """
-        super().__init__(
-            precursors=precursors, targets=targets, calculators=calculators
-        )
+
+        super().__init__(precursors=precursors, targets=targets)
 
         self.n = n
         self.exclusive_precursors = exclusive_precursors
         self.exclusive_targets = exclusive_targets
+        self.filter_by_chemsys = filter_by_chemsys
+        self.max_num_constraints = max_num_constraints
         self.remove_unbalanced = remove_unbalanced
         self.remove_changed = remove_changed
         self.calculate_e_above_hulls = calculate_e_above_hulls
         self.quiet = quiet
-        self.parallel = parallel
 
         self._stabilize = False
 
+        self._p_set_func = "issuperset" if self.exclusive_precursors else "intersection"
+        self._t_set_func = "issuperset" if self.exclusive_targets else "intersection"
+
         self.open_phases: Optional[List] = None
         self._build_pd = False
         self._build_grand_pd = False
 
-    def enumerate(self, entries: GibbsEntrySet) -> List[ComputedReaction]:
+    def enumerate(self, entries: GibbsEntrySet, batch_size=None) -> ReactionSet:
         """
-        Calculate all possible reactions given a set of entries. If the enumerator
-        was initialized with specified precursors or target, the reactions will be
-        filtered by these constraints. Every enumerator follows a standard procedure:
+        Calculate all possible reactions given a set of entries. If the enumerator was
+        initialized with specified precursors or target, the reactions will be filtered
+        by these constraints. Every enumerator follows a standard procedure:
 
         1. Initialize entries, i.e., ensure that precursors and target are considered
-        stable entries within the entry set. If using ChempotDistanceCalculator,
-        ensure that entries are filtered by stability.
+        stable entries within the entry set. If using ChempotDistanceCalculator, ensure
+        that entries are filtered by stability.
 
         2. Get a dictionary representing every possible "node", i.e. phase combination,
         grouped by chemical system.
@@ -124,83 +126,141 @@ class BasicEnumerator(Enumerator):
             entries: the set of all entries to enumerate from
         """
 
+        initialize_ray()
+
         entries, precursors, targets, open_entries = self._get_initialized_entries(
             entries
         )
 
-        combos_dict = self._get_combos_dict(entries, precursors, targets, open_entries)
+        combos_dict = self._get_combos_dict(
+            entries,
+            precursors,
+            targets,
+            open_entries,
+        )
         open_combos = self._get_open_combos(open_entries)
+
         if not open_combos:
             open_combos = []
 
         items = combos_dict.items()
 
-        parallel = False
-        median_comp_size = median([len(v) for v in combos_dict.values()])
-
-        if (
-            median_comp_size > PARALLEL_THRESHOLD
-            and len(combos_dict) > 1
-            and self.parallel
-        ):
-            parallel = True
-
-            initialize_ray()
-            obj = ray.put(self)
-            open_combos = ray.put(open_combos)
-            entries = ray.put(entries)
-            open_entries = ray.put(open_entries)
-            precursors = ray.put(precursors)
-            targets = ray.put(targets)
-
-            rxns = [
-                _get_rxns_from_chemsys_ray.remote(
-                    obj, item, open_combos, entries, open_entries, precursors, targets
-                )
-                for item in items
-            ]
-        else:
-            rxns = []
-
-            for item in tqdm(items, disable=self.quiet):
-                rxns.append(
-                    self._get_rxns_in_chemsys(
-                        item, open_combos, entries, open_entries, precursors, targets
-                    )
-                )
+        precursors = ray.put(precursors)
+        targets = ray.put(targets)
+        react_function = ray.put(self._react_function)
+        open_entries = ray.put(open_entries)
+        p_set_func = ray.put(self._p_set_func)
+        t_set_func = ray.put(self._t_set_func)
+        remove_unbalanced = ray.put(self.remove_unbalanced)
+        remove_changed = ray.put(self.remove_changed)
+        max_num_constraints = ray.put(self.max_num_constraints)
 
+        rxn_chunk_refs = []  # type: ignore
         results = []
 
-        if parallel:
-            iterator = to_iterator(rxns)
-            if not self.quiet:
-                iterator = tqdm(iterator, total=len(rxns), disable=self.quiet)
-        else:
-            iterator = rxns
+        if not batch_size:
+            batch_size = ray.cluster_resources()["CPU"] * 2
 
-        for r in iterator:
-            results.extend(r)
+        with tqdm(
+            total=self._num_chunks(items, open_combos), disable=self.quiet
+        ) as pbar:
+            for item in items:
+                chemsys, combos = item
 
-        return list(set(results))
+                elems = chemsys.split("-")
 
-    def estimate_max_num_reactions(self, entries: List[ComputedEntry]) -> int:
-        """
-        Estimate the upper bound of the number of possible reactions. This will
-        correlate with the amount of time it takes to enumerate reactions.
+                filtered_entries = None
+                pd = None
+                grand_pd = None
 
-        Args:
-            entries: A list of all entries to consider
+                if self.build_pd or self.build_grand_pd:
+                    filtered_entries = entries.get_subset_in_chemsys(elems)
 
-        Returns: The upper bound on the number of possible reactions
-        """
-        return sum([comb(len(entries), i) for i in range(1, self.n + 1)]) ** 2
+                if self.build_pd:
+                    pd = PhaseDiagram(filtered_entries)
+
+                if self.build_grand_pd:
+                    chempots = getattr(self, "chempots")
+                    grand_pd = GrandPotentialPhaseDiagram(filtered_entries, chempots)
+
+                filtered_entries = ray.put(filtered_entries)
+                pd = ray.put(pd)
+                grand_pd = ray.put(grand_pd)
+
+                for rxn_iterable_chunk in grouper(
+                    self._get_rxn_iterable(combos, open_combos), self.CHUNK_SIZE
+                ):
+                    if len(rxn_chunk_refs) > batch_size:
+                        num_ready = len(rxn_chunk_refs) - batch_size
+                        newly_completed, rxn_chunk_refs = ray.wait(
+                            rxn_chunk_refs, num_returns=num_ready
+                        )
+                        for completed_ref in newly_completed:
+                            results.extend(ray.get(completed_ref))
+
+                            pbar.update(1)
+
+                    rxn_chunk_refs.append(
+                        _react.remote(
+                            rxn_iterable_chunk,
+                            react_function,
+                            open_entries,
+                            precursors,
+                            targets,
+                            p_set_func,
+                            t_set_func,
+                            remove_unbalanced,
+                            remove_changed,
+                            max_num_constraints,
+                            filtered_entries,
+                            pd,
+                            grand_pd,
+                        )
+                    )
+
+            newly_completed, rxn_chunk_refs = ray.wait(
+                rxn_chunk_refs, num_returns=len(rxn_chunk_refs)
+            )
+            for completed_ref in newly_completed:
+                results.extend(ray.get(completed_ref))
+                pbar.update(1)
+
+        all_indices, all_coeffs, all_data = [], [], []
+        for r in results:
+            all_indices.append(r[0])
+            all_coeffs.append(r[1])
+            all_data.append(r[2])
+
+        rxn_set = ReactionSet(
+            entries.entries_list, all_indices, all_coeffs, all_data=all_data
+        )
+        rxn_set = rxn_set.filter_duplicates()
+
+        return rxn_set
+
+    @classmethod
+    def _num_chunks(cls, items, open_combos):
+        _ = open_combos  # not used
+
+        n = 0
+        for _, i in items:
+            num_combos = cls._rxn_iter_length(i, open_combos)
+            if num_combos > 0:
+                n += num_combos // cls.CHUNK_SIZE + 1
+
+        return n
+
+    @staticmethod
+    def _rxn_iter_length(combos, open_combos):
+        _ = open_combos  # not used
+        return comb(len(combos), 2)
 
     def _get_combos_dict(
         self, entries, precursor_entries, target_entries, open_entries
     ):
         """
-        Gets all possible entry combinations up to predefined cardinality n, filtered and
-        grouped by chemical system
+        Gets all possible entry combinations up to predefined cardinality (n), filtered
+        and grouped by chemical system.
         """
         precursor_elems = [
             [str(el) for el in e.composition.elements] for e in precursor_entries
@@ -208,164 +268,40 @@ class BasicEnumerator(Enumerator):
         target_elems = [
             [str(el) for el in e.composition.elements] for e in target_entries
         ]
-        open_elems = [[str(el) for el in e.composition.elements] for e in open_entries]
+        all_open_elems = {el for e in open_entries for el in e.composition.elements}
 
         entries = entries - open_entries
 
         combos = [set(c) for c in limited_powerset(entries, self.n)]
-        combos_dict = group_by_chemsys(combos)
+        combos_dict = group_by_chemsys(combos, all_open_elems)
 
         filtered_combos = self._filter_dict_by_elems(
-            combos_dict, precursor_elems, target_elems, open_elems
+            combos_dict,
+            precursor_elems,
+            target_elems,
+            all_open_elems,
         )
 
         return filtered_combos
 
-    def _get_open_combos(  # pylint: disable=R1711
+    def _get_open_combos(  # pylint: disable=useless-return
         self, open_entries
     ) -> Optional[List[Set[ComputedEntry]]]:
         """No open entries for BasicEnumerator, returns None"""
         _ = (self, open_entries)  # unused_arguments
         return None
 
-    def _get_rxns_in_chemsys(
-        self,
-        item,
-        open_combos,
-        entries,
-        open_entries,
-        precursors,
-        targets,
-    ):
-        """ """
-        chemsys, combos = item
-
-        elems = chemsys.split("-")
-        filtered_entries = entries.get_subset_in_chemsys(elems)
-        calculators = initialize_calculators(self.calculators, filtered_entries)
-
-        rxn_iter = self._get_rxn_iterable(combos, open_combos)
-
-        rxns = self._get_rxns_from_iterable(
-            rxn_iter,
-            precursors,
-            targets,
-            calculators,
-            filtered_entries,
-            open_entries,
-        )
-        return rxns
-
-    def _get_rxns_from_iterable(
-        self,
-        rxn_iterable,
-        precursors,
-        targets,
-        calculators,
-        filtered_entries=None,
-        open_entries=None,
-    ):
-        """
-        Returns reactions from an iterable representing the combination of 2 sets
-        of phases. Works for reactions with open phases.
-        """
-        pd = None
-        if self.build_pd:
-            pd = PhaseDiagram(filtered_entries)
-
-        grand_pd = None
-        if self.build_grand_pd:
-            chempots = getattr(self, "chempots")
-            grand_pd = GrandPotentialPhaseDiagram(filtered_entries, chempots)
-
-        p_set_func = "issuperset" if self.exclusive_precursors else "intersection"
-        t_set_func = "issuperset" if self.exclusive_targets else "intersection"
-
-        if not open_entries:
-            open_entries = set()
-
-        rxns = []
-
-        for reactants, products in rxn_iterable:
-            r = set(reactants) if reactants else set()
-            p = set(products) if products else set()
-            all_phases = r | p
-
-            precursor_func = (
-                getattr(precursors | open_entries, p_set_func)
-                if precursors
-                else lambda e: True
-            )
-            target_func = (
-                getattr(targets | open_entries, t_set_func)
-                if targets
-                else lambda e: True
-            )
-
-            if (
-                (r & p)
-                or (precursors and not precursors & all_phases)
-                or (p and targets and not targets & all_phases)
-            ):
-                continue
-
-            if not (precursor_func(r) or (p and precursor_func(p))):
-                continue
-            if p and not (target_func(r) or target_func(p)):
-                continue
-
-            suggested_rxns = self._react(
-                r, p, calculators, filtered_entries, pd, grand_pd
-            )
-
-            for rxn in suggested_rxns:
-                if (
-                    rxn.is_identity
-                    or (self.remove_unbalanced and not rxn.balanced)
-                    or (self.remove_changed and rxn.lowest_num_errors != 0)
-                ):
-                    continue
-
-                reactant_entries = set(rxn.reactant_entries) - open_entries
-                product_entries = set(rxn.product_entries) - open_entries
-
-                if precursor_func(reactant_entries) and target_func(product_entries):
-                    rxns.append(rxn)
-
-        return rxns
-
-    def _react(
-        self,
-        reactants,
-        products,
-        calculators,
-        filtered_entries=None,
-        pd=None,
-        grand_pd=None,
-    ):
-        """
-        Generates reactions from a list of reactants, products, and optional
-        calculator(s)
-        """
-        _ = (
-            filtered_entries,
-            pd,
-            grand_pd,
-            self,
-        )  # unused arguments in BasicEnumerator class
-
+    @staticmethod
+    def _react_function(reactants, products, **kwargs):
+        _ = kwargs  # unused_argument
         forward_rxn = ComputedReaction.balance(reactants, products)
         backward_rxn = forward_rxn.reverse()
-
-        forward_rxn = apply_calculators(forward_rxn, calculators)
-        backward_rxn = apply_calculators(backward_rxn, calculators)
-
         return [forward_rxn, backward_rxn]
 
     @staticmethod
     def _get_rxn_iterable(combos, open_combos):
         """Get all reaction/product combinations"""
-        _ = open_combos  # unused argument in BasicEnumerator class
+        _ = open_combos  # unused argument
 
         return combinations(combos, 2)
 
@@ -401,12 +337,16 @@ class BasicEnumerator(Enumerator):
 
         if self.stabilize:
             entries_new = entries_new.filter_by_stability(e_above_hull=0.0)
-            self.logger.info("Filtering by stable entries!")
+            logger.info("Filtering by stable entries!")
+
+        entries_new.build_indices()
 
         open_entries = set()
         if self.open_phases:
             open_entries = {
-                e for e in entries if e.composition.reduced_formula in self.open_phases
+                e
+                for e in entries_new
+                if e.composition.reduced_formula in self.open_phases
             }
 
         return entries_new, precursors, targets, open_entries
@@ -416,41 +356,40 @@ class BasicEnumerator(Enumerator):
         combos_dict,
         precursor_elems,
         target_elems,
-        open_elems,
+        all_open_elems,
     ):
         """Filters the dictionary of combinations by elements"""
         filtered_dict = {}
 
         all_precursor_elems = {el for g in precursor_elems for el in g}
         all_target_elems = {el for g in target_elems for el in g}
+        all_open_elems = {str(el) for el in all_open_elems}
+
+        filter_elems = None
+        if self.filter_by_chemsys:
+            filter_elems = set(self.filter_by_chemsys.split("-"))
 
         for chemsys, combos in combos_dict.items():
             elems = set(chemsys.split("-"))
 
-            if len(elems) >= 10 or len(elems) == 1:
-                continue
+            if filter_elems:
+                if not elems.issuperset(filter_elems):
+                    continue
 
-            all_open_elems = {el for g in open_elems for el in g}
-            if not all_open_elems.issubset(elems):
+            if len(elems) >= 10 or len(elems) == 1:  # too few or too many elements
                 continue
 
             if precursor_elems:
-                if self.exclusive_precursors:
-                    if not all_precursor_elems == elems:
-                        continue
-                else:
-                    if not any(
-                        elems.issuperset(el_group) for el_group in precursor_elems
-                    ):
-                        continue
+                if not getattr(all_precursor_elems | all_open_elems, self._p_set_func)(
+                    elems
+                ):
+                    continue
 
             if target_elems:
-                if self.exclusive_targets:
-                    if not all_target_elems == elems:
-                        continue
-                else:
-                    if not any(elems.issuperset(el_group) for el_group in target_elems):
-                        continue
+                if not getattr(all_target_elems | all_open_elems, self._t_set_func)(
+                    elems
+                ):
+                    continue
 
             filtered_dict[chemsys] = combos
 
@@ -482,20 +421,22 @@ class BasicOpenEnumerator(BasicEnumerator):
     the ReactionSet class).
     """
 
+    CHUNK_SIZE = 2500
+
     def __init__(
         self,
         open_phases: List[str],
         precursors: Optional[List[str]] = None,
         targets: Optional[List[str]] = None,
-        calculators: Optional[List[str]] = None,
         n: int = 2,
         exclusive_precursors: bool = True,
         exclusive_targets: bool = False,
+        filter_by_chemsys: Optional[str] = None,
+        max_num_constraints: int = 1,
         remove_unbalanced: bool = True,
         remove_changed: bool = True,
         calculate_e_above_hulls: bool = False,
         quiet: bool = False,
-        parallel: bool = True,
     ):
         """
         Supplied target and calculator parameters are automatically initialized as
@@ -505,8 +446,8 @@ class BasicOpenEnumerator(BasicEnumerator):
             open_phases: List of formulas of open entries (e.g. ["O2"])
             precursors: Optional list of formulas of precursor phases; only reactions
                 which have these phases as reactants will be enumerated.
-            targets: Optional list of formulas of targets; only reactions which make this target
-                will be enumerated.
+            targets: Optional list of formulas of targets; only reactions
+                which make this target will be enumerated.
             calculators: Optional list of Calculator object names; see calculators
                 module for options (e.g., ["ChempotDistanceCalculator]).
             n: Maximum reactant/product cardinality; i.e., largest possible number of
@@ -525,38 +466,23 @@ class BasicOpenEnumerator(BasicEnumerator):
         super().__init__(
             precursors=precursors,
             targets=targets,
-            calculators=calculators,
             n=n,
             exclusive_precursors=exclusive_precursors,
             exclusive_targets=exclusive_targets,
+            filter_by_chemsys=filter_by_chemsys,
+            max_num_constraints=max_num_constraints,
             remove_unbalanced=remove_unbalanced,
             remove_changed=remove_changed,
-            quiet=quiet,
-            parallel=parallel,
         )
         self.open_phases: List[str] = open_phases
 
-    def estimate_max_num_reactions(self, entries: List[ComputedEntry]) -> int:
-        """
-        Estimate the upper bound of the number of possible reactions. This will
-        correlate with the amount of time it takes to enumerate reactions.
-
-        Args:
-            entries: A list of all entries to consider
-
-        Returns: The upper bound on the number of possible reactions
-        """
-        num_open_phases = len(self.open_phases)
+    @staticmethod
+    def _rxn_iter_length(combos, open_combos):
         num_combos_with_open = sum(
-            [comb(num_open_phases, i) for i in range(1, num_open_phases + 1)]
+            1 if not i & j else 0 for i in combos for j in open_combos
         )
 
-        num_total_combos = 0
-        for i in range(1, self.n + 1):
-            num_combos = comb(len(entries), i)
-            num_total_combos += num_combos_with_open * num_combos
-
-        return num_total_combos**2
+        return len(combos) * num_combos_with_open
 
     def _get_open_combos(self, open_entries):
         """Get all possible combinations of open entries. For a single entry,
@@ -581,13 +507,82 @@ class BasicOpenEnumerator(BasicEnumerator):
 
 
 @ray.remote
-def _get_rxns_from_chemsys_ray(
-    obj, item, open_combos, entries, open_entries, precursors, targets
+def _react(
+    rxn_iterable,
+    react_function,
+    open_entries,
+    precursors,
+    targets,
+    p_set_func,
+    t_set_func,
+    remove_unbalanced,
+    remove_changed,
+    max_num_constraints,
+    filtered_entries,
+    pd,
+    grand_pd,
 ):
     """
-    Wrapper function for parallelizing reaction enumeration using ray. See
-    _get_rxns_in_chemsys in BasicEnumerator for more details.
+    This function is a wrapper for the specific react function of each enumerator. This
+    wrapper contains the logic used for filtering out reactions based on the
+    user-defined enumerator settings. It can also be called as a remote function using
+    ray, allowing for parallel computation during reaction enumeration.
+
+    Note: this function is not intended to to be called directly!
+
     """
-    return obj._get_rxns_in_chemsys(  # pylint: disable=protected-access
-        item, open_combos, entries, open_entries, precursors, targets
-    )
+    all_rxns = []
+
+    for rp in rxn_iterable:
+        if not rp:
+            continue
+
+        r = set(rp[0]) if rp[0] else set()
+        p = set(rp[1]) if rp[1] else set()
+
+        all_phases = r | p
+
+        precursor_func = (
+            getattr(precursors | open_entries, p_set_func)
+            if precursors
+            else lambda e: True
+        )
+        target_func = (
+            getattr(targets | open_entries, t_set_func) if targets else lambda e: True
+        )
+
+        if (
+            (r & p)
+            or (precursors and not precursors & all_phases)
+            or (p and targets and not targets & all_phases)
+        ):
+            continue
+
+        if not (precursor_func(r) or (p and precursor_func(p))):
+            continue
+        if p and not (target_func(r) or target_func(p)):
+            continue
+
+        suggested_rxns = react_function(
+            r, p, filtered_entries=filtered_entries, pd=pd, grand_pd=grand_pd
+        )
+
+        rxns = []
+        for rxn in suggested_rxns:
+            if (
+                rxn.is_identity
+                or (remove_unbalanced and not rxn.balanced)
+                or (remove_changed and rxn.lowest_num_errors != 0)
+                or rxn.data["num_constraints"] > max_num_constraints
+            ):
+                continue
+
+            reactant_entries = set(rxn.reactant_entries) - open_entries
+            product_entries = set(rxn.product_entries) - open_entries
+
+            if precursor_func(reactant_entries) and target_func(product_entries):
+                rxns.append(get_rxn_info(rxn))
+
+        all_rxns.extend(rxns)
+
+    return all_rxns
